@@ -670,7 +670,7 @@ static inline void xauth_write_string(int fd,const char *s)
 	write(fd,s,len);
 }
 
-static void xauth_write_file(const char *file,int dpy,char data[16])
+static int xauth_write_file(const char *file,int dpy,char data[16])
 {
 	int fd;
 	char addr[128];
@@ -680,7 +680,7 @@ static void xauth_write_file(const char *file,int dpy,char data[16])
 	gethostname(addr,sizeof(addr));
 	
 	fd=open(file,O_CREAT|O_TRUNC|O_WRONLY,0600);
-	if(fd==-1) return;
+	if(fd==-1) return -1;
 	xauth_write_uint16(fd,256);		//FamilyLocalHost
 	xauth_write_string(fd,addr);
 	xauth_write_string(fd,buf);
@@ -688,6 +688,7 @@ static void xauth_write_file(const char *file,int dpy,char data[16])
 	xauth_write_uint16(fd,16);
 	write(fd,data,16);
 	close(fd);
+	return 0;
 }
 
 static void create_server_auth(LXSession *s)
@@ -695,6 +696,7 @@ static void create_server_auth(LXSession *s)
 	GRand *h;
 	int i;
 	char *authfile;
+	struct passwd *pw;
 
 	h = g_rand_new();
 	for( i = 0; i < 16; i++ )
@@ -708,6 +710,11 @@ static void create_server_auth(LXSession *s)
 	setenv("XAUTHORITY",authfile,1);
 	remove(authfile);
 	xauth_write_file(authfile,s->display,s->mcookie);
+	pw=getpwnam("lxdm");endpwent();
+	if(pw!=NULL)
+	{
+		chown(authfile,pw->pw_uid,pw->pw_gid);
+	}
 	g_free(authfile);
 }
 
@@ -715,9 +722,6 @@ static char ** create_client_auth(struct passwd *pw,char **env)
 {
 	LXSession *s;
 	char *authfile;
-	
-	if(pw->pw_uid==0) /* root don't need it */
-		return env;
         
 	s=lxsession_find_user(pw->pw_uid);
 	if(!s)
@@ -743,7 +747,13 @@ static char ** create_client_auth(struct passwd *pw,char **env)
 		}
 	}
 	remove(authfile);
-	xauth_write_file(authfile,s->display,s->mcookie);
+	if(xauth_write_file(authfile,s->display,s->mcookie)==-1)
+	{
+		g_free(authfile);
+		authfile = g_strdup_printf("/var/run/lxdm/.Xauth%d",pw->pw_uid);
+		remove(authfile);
+		xauth_write_file(authfile,s->display,s->mcookie);
+	}
 	env=g_environ_setenv(env,"XAUTHORITY",authfile,TRUE);
 	chown(authfile,pw->pw_uid,pw->pw_gid);
 	g_free(authfile);
@@ -1009,9 +1019,6 @@ static void lxdm_startx(LXSession *s)
 	g_message("%ld: start xserver in %d retry",time(NULL),i);
 	if(s->dpy==NULL)
 		exit(EXIT_FAILURE);
-	#ifndef DISABLE_XAUTH
-	unsetenv("XAUTHORITY");
-	#endif
 	
 	if(s->option && g_key_file_has_key(config,s->option,"numlock",NULL))
 	{
@@ -1028,6 +1035,9 @@ static void lxdm_startx(LXSession *s)
 		g_spawn_command_line_async(arg,NULL);
 		g_free(arg);
 	}
+	#ifndef DISABLE_XAUTH
+	unsetenv("XAUTHORITY");
+	#endif
 }
 
 static void exit_cb(void)
@@ -1402,6 +1412,7 @@ void lxdm_do_login(struct passwd *pw, char *session, char *lang, char *option)
 	if( session_desktop_names && session_desktop_names[0] )
 		env=g_environ_setenv(env, "XDG_CURRENT_DESKTOP", session_desktop_names, TRUE);
 	
+	env=lxdm_auth_append_env(&s->auth,env);
 #ifndef DISABLE_XAUTH
 	env=create_client_auth(pw,env);
 #endif
